# A simulation of grass, rabbits, and a virus.
#
# This code is in the public domain.

import math
import sys

from matplotlib import pyplot
import numpy


class LV:
	"""A Lotka-Volterra-like model of grass and rabbits.

	Construct with initial numbers for grass and rabbits; grass can't
	be larger than resource_limit.
	"""
	def __init__(self, grass, rabbits, resource_limit=1000):
		self.grass, self.rabbits = grass, rabbits
		self.resource_limit = resource_limit

	def get_growth_factor(self):
		"""returns the factor of growth for the rabbits.

		This is a concatenation of simple, slightly non-linear functions
		here, where the population is constant when grass=rabbits.
		"""
		grass_per_rabbits = self.grass/self.rabbits
		if grass_per_rabbits<1:
			return grass_per_rabbits**2
		else:
			return 1+math.sqrt(grass_per_rabbits-1)

	def propagate(self):
		"""runs one time step for the model.
		"""
		self.rabbits *= self.get_growth_factor()
		self.grass = min(2*self.grass, self.resource_limit)
		self.grass = max(1, self.grass-self.rabbits)

	def get_populations(self, pre_iter=1000, log_iter=100):
		"""returns log_iter pairs of rabbits, grass.

		The system is first run pre_iter steps to relax from the initial
		conditions.
		"""
		for round in range(pre_iter):
			self.propagate()

		result = []
		for round in range(log_iter):
			self.propagate()
			result.append((self.rabbits, self.grass))
		return result
			

class LVWithVirus(LV):
	"""This is LV, just with a virus with a given deadliness.

	The model simply is: All rabbits meet once per round, and they
	die with deadliness after such an encounter.
	"""
	def __init__(self, grass, rabbits, deadliness, **kwargs):
		LV.__init__(self, grass, rabbits, **kwargs)
		self.deadliness = deadliness

	def propagate(self):
		LV.propagate(self)
		self.rabbits = max(1, self.rabbits-self.rabbits**2*self.deadliness)


def main():
	pars = [float(s) for s in sys.argv[1:]]
	if len(pars)==3:
		model = LVWithVirus(*pars)
	elif len(pars)==2:
		model = LV(*pars)
	else:
		sys.exit(f"{sys.argv[0]} grass rabbits [deadliness]")

	pops = numpy.array(model.get_populations())

	fig, ax = pyplot.subplots(figsize=(10,4))
	ax.plot(pops[:,0], label="rabbits")
	ax.plot(pops[:,1], label="grass")
	ax.legend(loc='upper center')
	pyplot.savefig("pops.svg")
	pyplot.show()


if __name__=="__main__":
	main()
